%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% stereo.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% function [d] = stereo(corrL, corrR)
%
% CS223b
% Winter '05
%
% Compute disparity from two rectified grayscale images.  Input parameters
% are rectified left and right images.
%
% Adapted by Dan Morris from:
% http://robots.stanford.edu/cs223b04/homework/hw3/hw3_sol.pdf
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [d] = stereo(corrL, corrR)

% The maximum allowable disparity...
max_d = 10;

% The _radius_ of the search window... a radius of 2, for example,
% gives a 5x5 window
rad = 3;

% Figure out the iimage and window size
[rows cols] = size(corrL);
imL = zeros(rows+(2*rad),cols+(2*rad));
imR = zeros(rows+(2*rad),cols+(2*rad));
imL(rad+1:rad+rows,rad+1:rad+cols) = corrL;
imR(rad+1:rad+rows,rad+1:rad+cols) = corrR;

% Initialize the disparity matrix
d = zeros(size(imL));

% For each row...
for u = rad+1:rows+rad
  
  % For each column we can search...
  for v=max_d+rad+1:cols+rad
  
  % The 2*rad x 2*rad + 1 window we want to _find_ in the right image
  smL = imL(u-rad:u+rad,v-rad:v+rad);
    
  % A vector of correlation values; the minimum will be our winner...
  %
  % So we initialize it to the largest possible value...
  vals = 99e9*ones(1,cols+2*rad);
  
  % Search back to the left in the right image... 
  for mv=v-max_d+1:v
      
    % The current window in the right image...
    smR = imR(u-rad:u+rad,mv-rad:mv+rad);
      
    % The correlation value for this window...
    vals(mv) = sum(sum((smL-smR).^2));        
    
  end
  
  % The best correlation value and the corresponding disparity
  [mn, dp] = min(vals);
  
  % Put this in the output vector
  d(u,v) = v-dp;
  end
  
end

% Clip away the useless parts of our disparity map
d = d(rad+1:rad+rows, rad+1:rad+cols);
